

/* This Program tests many functions of the BASTLI AVR Demoboard. It can be used as a basis for beginners. 
   For help and assistance - visit www.bastli.ethz.ch  and/or consult the documentation for the board.

   Code written by Mario Mauerer, student at the ITET. Target device: AVR ATmega32, running at 16MHz.
*/

#include <avr/io.h>  
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include "AVR_Demoboard_Testprogramm_Header.h" //Includes the Header-File where the function prototypes and #defines etc. are located.


//global variables: 
volatile uint8_t TMR0OVF_flag=0; //Timer0 Interrupt-Flag. (ISR sets Flag)
volatile uint8_t TMR1COMPA_flag=0; //Timer 1 Compare Match-Flag (ISR sets Flag)
volatile uint8_t TMR2upcntr=0;

int main(void)
{

	IO_init();
	peripheral_init();
	

	uint8_t number = 0; //Number to be displayed on the seven-segment-display
	uint16_t TMR0counter = 0; //Counter for the Timer0 - Increased every time Timer0 overflows. 

	while(1)
	{
		
		if(TMR0OVF_flag == 1) //If the Timer0 Overflow-Interrupt has been triggered
		{
			TMR0OVF_flag = 0;
			TMR0counter++;
			
			if(TMR0counter == 6)
			{
				TMR0counter = 0;	
				LED_statemachine_update();
				
			}	
		}
		
		if(TMR1COMPA_flag == 1)
		{
			TMR1COMPA_flag = 0;

			number++;
			if(number == 16)
			number = 0;

			seven_segm_set(number);
		}

		if(TMR2upcntr == 10)
		{
			TMR2upcntr = 0;	
			TCCR2 &= ~((1<<CS22) | (1<<CS21) | (1<<CS20)); //Stop Timer2
			TIMSK |= (1<<TOIE0); //Timer0 Overflow Interrupt Enable
			TIMSK |= (1<<OCIE1A); //Enable Compare1A-Interrupt

		}


		if(!(PINA & (1<<0))) //if switch 2 pressed
		{
			UART_send("Switch 2 pressed");

		}
		if(!(PINA & (1<<1))) //if switch 3 pressed
		{
			UART_send("Switch 3 pressed");

		}
		if(!(PINA & (1<<2))) //if switch 4 pressed
		{
			UART_send("Switch 4 pressed");

		}
		if(!(PINA & (1<<3))) //if switch 5 pressed
		{
			static char text[6];
			utoa(get_adc(),text,10);
			UART_send(text);

		}

		if(PINB == 0b00011000)
		{
			UART_send("DILswitch set!");

		}

		
	}

}


void IO_init(void) //This procedure sets up the IO-pins of the ATmega32.
{

	DDRD |= ((1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7)); //LED-Pins = outputs
	PORTD |= ((1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7)); //shut every LED off (LEDs are active-low)

	DDRC |= ((1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7)); //7-segment-display: output-pins
	PORTC &= ~((1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7)); //shut every digit off(display is active-high)

	DDRB &= ~((1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7)); //DIL-Switch: Inputs. (Switch is active-High ==> no pullups)

	DDRA &= ~((1<<0) | (1<<1) | (1<<2) | (1<<3)); //Print-Switch-Inputs. active low==> enable Pullups.
	PORTA |= ((1<<0) | (1<<1) | (1<<2) | (1<<3)); //Pullups enabled

	DDRA &= ~(1<<7); //NTC Input (ADC7) - no pullup!

}

void peripheral_init(void) //This procedure initializes some peripheral features like timers or interfaces(i.e. UART) of the ATmega32.
{
	//Initialize the USART:
	UCSRB |= (1<<RXEN); //Receiver enable
	UCSRB |= (1<<TXEN); //Transmitter enable
	UCSRC |= ((1<<UCSZ0) | (1<<UCSZ1)); //8 Data Bits, 1 Stop Bit, Asynchronous
	UBRRH = 0; //Write UBRRH Register
	UBRRL = 103; //9600 Baud //Write UBRRL Register

	//Initialize Timer0 as simple Overflow-Interrupt-Counter
	TCCR0 |= ((1<<CS02) | (1<<CS00)); //Prescaler = 1024 - Start Counter 0
	TIMSK |= (1<<TOIE0); //Timer0 Overflow Interrupt Enable

	//Initialize Timer1 as "Clear Timer on Compare Match" Interrupt Timer.
	TCCR1B |= (1<<WGM12); //CTC Mode
	TCCR1B |= ((1<<CS10) | (1<<CS11)); //Prescaler = 64
	OCR1A = 50000;
	TIMSK |= (1<<OCIE1A); //Enable Compare1A-Interrupt

	//Configure External Interrupt INT0:
	MCUCR |= (1<<ISC00); //Any Logic Change triggers the INT0-Interrupt
	GICR |= (1<<INT0); //Enable INT0;

	//Configure Timer2:
	//TCCR2 |= ((1<<CS22) | (1<<CS21) | (1<<CS20)); //Start Timer2
	TIMSK |= (1<<TOIE2); //Timer2 OVF-Interrupt

	//Configure ADC:
	ADMUX |= (1<<REFS0); //Voltage Reference = AVCC
	ADMUX |= ((1<<0) | (1<<1) | (1<<2)); //ADC Input = Single Ended ADC 7
	ADCSRA |= ((1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0)); //ADC Prescaler = 128 ==> 125kHz ADC freq.
	ADCSRA |= (1<<ADEN); //ADC Enable
	ADCSRA |= (1<<ADATE); //ADC auto trigger enable
	ADCSRA |= (1<<ADSC); //Start free running mode


	SREG |= (1<<7); //Enable Interrupts globally
}

void seven_segm_set(uint8_t number) //input: the number to be displayed. (0 to 15 - Hex) 
{
	PORTC &= ~((1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7)); //shut every digit off(display is active-high)
	
	switch(number)
	{
	case 0: PORTC |= ((1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6));
			break;
	
	case 1:	PORTC |= ((1<<2) | (1<<3));
			break;
	
	case 2: PORTC |= ((1<<1) | (1<<2) | (1<<7) | (1<<5) | (1<<4));
			break;

	case 3: PORTC |= ((1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<7));
			break;

	case 4: PORTC |= ((1<<2) | (1<<6) | (1<<7)  | (1<<3));
			break;

	case 5: PORTC |= ((1<<1) | (1<<6) | (1<<7) | (1<<3) | (1<<4));
			break;

	case 6: PORTC |= ((1<<1) | (1<<6) | (1<<7) | (1<<3) | (1<<4) | (1<<5));
			break;

	case 7: PORTC |= ((1<<1) | (1<<2) | (1<<3));
			break;

	case 8: PORTC |= ((1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7));
			break;

	case 9: PORTC |= ((1<<7) | (1<<6) | (1<<1) | (1<<2) | (1<<3) | (1<<4));
			break;

	case 0xA: PORTC |= ((1<<1) | (1<<2) | (1<<3) | (1<<5) | (1<<6) | (1<<7));
			  break;

	case 0xB: PORTC |= ((1<<6) | (1<<3) | (1<<4) | (1<<5) | (1<<7));
			  break;

	case 0xC: PORTC |= ((1<<1) | (1<<4) | (1<<5) | (1<<6));
			  break;

	case 0xD: PORTC |= ((1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<7));
			  break;

	case 0xE: PORTC |= ((1<<1) | (1<<4) | (1<<5) | (1<<6) | (1<<7));
			  break;

	case 0xF: PORTC |= ((1<<1) | (1<<5) | (1<<6) | (1<<7));
			  break;
	}

}

void LED_statemachine_update(void)
{
	static uint8_t ledstate = 0;
	
	ledstate++;
	if(ledstate == 8)
	{
		ledstate = 0;
	}

	PORTD |= ((1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7)); //shut every LED off (LEDs are active-low)

	switch(ledstate)
	{
		case 0: PORTD &= ~(1<<3);
				break;
		case 1: PORTD &= ~(1<<4);
				break;
		case 2: PORTD &= ~(1<<5);
				break;
		case 3: PORTD &= ~(1<<6);
				break;
		case 4: PORTD &= ~(1<<7);
				break;
		case 5: PORTD &= ~(1<<6);
				break;
		case 6: PORTD &= ~(1<<5);
				break;
		case 7: PORTD &= ~(1<<4);
				break;
	}

}

uint16_t get_adc(void)
{
	return ADC;
}

void UART_send(char *text)
{

	while(*text) //End of string = 0
	{
		while (!(UCSRA & (1<<UDRE))) // Wait until Buffer is empty - ready to send.
		{
		}

		UDR = *text; 

		text++;
	}

	//Next line in the terminal:

	while (!(UCSRA & (1<<UDRE))) // Wait until Buffer is empty - ready to send.
	{
	}
	UDR = 0x0D; //Carriage Return

	while (!(UCSRA & (1<<UDRE))) // Wait until Buffer is empty - ready to send.
	{
	}
	UDR = 0x0A; //Line Feed

}

//*****INTERRUPT SERVICE ROUTINES*****************

ISR(TIMER0_OVF_vect) //Timer0 Overflow Interrupt Service Routine
{
	TMR0OVF_flag = 1;
}

ISR(TIMER1_COMPA_vect)
{
	TMR1COMPA_flag = 1;
	PORTC ^= (1<<0); //Toggle the seven-segment-dot
}

ISR(INT0_vect)
{
	TIMSK &= ~(1<<TOIE0); //Timer0 Overflow Interrupt DISable
	TIMSK &= ~(1<<OCIE1A); //DISable Compare1A-Interrupt			

	TCNT2 = 0;
	TMR2upcntr = 0;	
	TCCR2 |= ((1<<CS22) | (1<<CS21) | (1<<CS20)); //Start Timer2

}

ISR(TIMER2_OVF_vect)
{
	TMR2upcntr++;

}
